"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_transformer_core_1 = require("graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const graphql_dynamodb_transformer_1 = require("graphql-dynamodb-transformer");
const ModelAuthTransformer_1 = require("../ModelAuthTransformer");
test('Test "read" auth operation', () => {
    const validSchema = `
    type Post @model @auth(rules: [{allow: groups, groups: ["Admin", "Dev"], operations: [read]}]) {
        id: ID!
        title: String!
        createdAt: String
        updatedAt: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toContain('Authorization rule:');
});
test('Test "create", "update", "delete" auth operations', () => {
    const validSchema = `
    type Post @model @auth(rules: [{allow: groups, groups: ["Admin", "Dev"], operations: [create, update, delete]}]) {
        id: ID!
        title: String!
        createdAt: String
        updatedAt: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.getPost.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Query.listPosts.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.createPost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.createPost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toMatchSnapshot();
});
test('Test that operation overwrites queries in auth operations', () => {
    const validSchema = `
    type Post @model @auth(rules: [{allow: groups, groups: ["Admin", "Dev"], queries: [get, list], operations: [create, update, delete]}]) {
        id: ID!
        title: String!
        createdAt: String
        updatedAt: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.getPost.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Query.listPosts.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.createPost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.createPost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toMatchSnapshot();
});
test('Test that checks subscription resolvers are generated with auth logic', () => {
    const validSchema = `
    type Salary @model
    @auth( rules: [
        {allow: owner},
        {allow: groups, groups: ["Admin"]}]){
        id: ID!
        wage: Int
        owner: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    // expect to generate subcriptions resolvers
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onDeleteSalary.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onDeleteSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).toMatchSnapshot();
});
test('Test that checks subscription resolvers are created without auth logic', () => {
    const validSchema = `
    type Salary @model(
        subscriptions: {
            level: public
        }) @auth( rules: [
        {allow: owner},
        {allow: groups, groups: ["Admin"]}]){
        id: ID!
        wage: Int
        owner: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    // expect to generate subcriptions resolvers
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onDeleteSalary.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onDeleteSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).toMatchSnapshot();
});
test('Test that subscriptions are only generated if the respective mutation operation exists', () => {
    const validSchema = `
    type Salary @model(
        mutations: {
            create: "makeIT",
            update: "updateIT",
        })@auth(
            rules: [
                {allow: owner},
                {allow: groups, groups: ["Moderator"], operations: [create]}
            ]){
        id: ID!
        wage: Int
        owner: String
    }
    `;
    const transformer = new graphql_transformer_core_1.GraphQLTransform({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({
                authConfig: {
                    defaultAuthentication: {
                        authenticationType: 'AMAZON_COGNITO_USER_POOLS',
                    },
                    additionalAuthenticationProviders: [],
                },
            }),
        ],
    });
    const out = transformer.transform(validSchema);
    // expect to generate subscription resolvers for create and update only
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onCreateSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Subscription.onUpdateSalary.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Subscription.onDeleteSalary.res.vtl']).toBeUndefined();
});
//# sourceMappingURL=OperationsArgument.test.js.map